<?php
/**
 * Plugin Name: WordPress Native Gallery Links on Captions
 * Plugin URI: https://iuliacazan.ro/apply-links-to-wordpress-native-gallery-captions/
 * Description: This plugin changes the elements order in the native gallery markup, to allow the click on the entire image.
 * Text Domain: ic-devops
 * Version: 1.0
 * Author: Iulia Cazan - PHP Coder
 * Author URI: https://iuliacazan.ro/
 * License: GPL2
 *
 * @package ic-devops
 *
 * Copyright (C) 2019 Iulia Cazan - PHP Coder
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License, version 2, as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

define( 'ICDEVOPS_FIX_GALLERY_LINK', 1.0 );

/**
 * Class for IC DevOps Fix Gallery Links
 */
class ICDevOps_Fix_Gallery_Links {

	/**
	 * Class instance.
	 *
	 * @var object
	 */
	private static $instance;

	/**
	 * Get active object instance.
	 *
	 * @return object
	 */
	public static function get_instance() {
		if ( ! self::$instance ) {
			self::$instance = new ICDevOps_Fix_Gallery_Links();
		}
		return self::$instance;
	}

	/**
	 * Class constructor, includes and init method.
	 *
	 * @return void
	 */
	public function __construct() {
		$this->init();
	}

	/**
	 * Run action and filter hooks.
	 *
	 * @return void
	 */
	private function init() {
		$called_class = get_called_class();
		add_filter( 'post_gallery', array( $called_class, 'replace_post_gallery_output' ), 10, 3 );
		add_action( 'wp_ajax_icdevops-gallery-caption-link-deactivate-notice', array( $called_class, 'admin_notices_cleanup' ) );

		add_action( 'admin_enqueue_scripts', array( $called_class, 'load_admin_assets' ) );
		add_action( 'wp_enqueue_scripts', array( $called_class, 'load_assets' ) );
		add_action( 'admin_notices', array( $called_class, 'admin_notices' ) );
		add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), array( $called_class, 'plugin_action_links' ) );
	}

	/**
	 * Enqueue the css and javascript files.
	 *
	 * @return void
	 */
	public static function load_admin_assets() {
		wp_enqueue_script(
			'fixgallink-admin',
			plugins_url( '/custom-admin.js', __FILE__ ),
			array( 'jquery' ),
			ICDEVOPS_FIX_GALLERY_LINK,
			true
		);
	}

	/**
	 * Enqueue the css and javascript files.
	 *
	 * @return void
	 */
	public static function load_assets() {
		wp_enqueue_script(
			'fixgallink',
			plugins_url( '/custom.js', __FILE__ ),
			array( 'jquery' ),
			ICDEVOPS_FIX_GALLERY_LINK,
			true
		);
	}

	/**
	 * Replace post gallery output.
	 *
	 * @param string  $output   The gallery output. Default empty.
	 * @param array   $atts     Attributes of the gallery shortcode.
	 * @param integer $instance Unique numeric ID of this gallery shortcode instance.
	 * @return string
	 */
	public static function replace_post_gallery_output( $output = '', $atts = null, $instance = null ) {
		if ( ! empty( $output ) ) {
			// Fail-fast.
			return $output;
		}

		$post  = get_post();
		$html5 = current_theme_supports( 'html5', 'gallery' );
		$atts  = shortcode_atts(
			array(
				'order'      => 'ASC',
				'orderby'    => 'menu_order ID',
				'id'         => $post ? $post->ID : 0,
				'itemtag'    => $html5 ? 'figure' : 'dl',
				'icontag'    => $html5 ? 'div' : 'dt',
				'captiontag' => $html5 ? 'figcaption' : 'dd',
				'columns'    => 3,
				'size'       => 'thumbnail',
				'include'    => '',
				'exclude'    => '',
				'link'       => '',
			),
			$atts,
			'gallery'
		);

		$id = intval( $atts['id'] );

		if ( ! empty( $atts['include'] ) ) {
			$_attachments = get_posts(
				array(
					'include'        => $atts['include'],
					'post_status'    => 'inherit',
					'post_type'      => 'attachment',
					'post_mime_type' => 'image',
					'order'          => $atts['order'],
					'orderby'        => $atts['orderby'],
				)
			);

			$attachments = array();
			foreach ( $_attachments as $key => $val ) {
				$attachments[ $val->ID ] = $_attachments[ $key ];
			}
		} elseif ( ! empty( $atts['exclude'] ) ) {
			$attachments = get_children(
				array(
					'post_parent'    => $id,
					'exclude'        => $atts['exclude'],
					'post_status'    => 'inherit',
					'post_type'      => 'attachment',
					'post_mime_type' => 'image',
					'order'          => $atts['order'],
					'orderby'        => $atts['orderby'],
				)
			);
		} else {
			$attachments = get_children(
				array(
					'post_parent'    => $id,
					'post_status'    => 'inherit',
					'post_type'      => 'attachment',
					'post_mime_type' => 'image',
					'order'          => $atts['order'],
					'orderby'        => $atts['orderby'],
				)
			);
		}

		if ( empty( $attachments ) ) {
			return '';
		}

		if ( is_feed() ) {
			$output = "\n";
			foreach ( $attachments as $att_id => $attachment ) {
				$output .= wp_get_attachment_link( $att_id, $atts['size'], true ) . "\n";
			}
			return $output;
		}

		$itemtag    = tag_escape( $atts['itemtag'] );
		$captiontag = tag_escape( $atts['captiontag'] );
		$icontag    = tag_escape( $atts['icontag'] );
		$valid_tags = wp_kses_allowed_html( 'post' );
		if ( ! isset( $valid_tags[ $itemtag ] ) ) {
			$itemtag = 'dl';
		}
		if ( ! isset( $valid_tags[ $captiontag ] ) ) {
			$captiontag = 'dd';
		}
		if ( ! isset( $valid_tags[ $icontag ] ) ) {
			$icontag = 'dt';
		}

		$columns   = intval( $atts['columns'] );
		$itemwidth = $columns > 0 ? floor( 100 / $columns ) : 100;
		$float     = is_rtl() ? 'right' : 'left';

		$selector = "gallery-{$instance}";

		$gallery_style = '';

		/**
		 * Filters whether to print default gallery styles.
		 *
		 * @since 3.1.0
		 *
		 * @param bool $print Whether to print default gallery styles.
		 *                    Defaults to false if the theme supports HTML5 galleries.
		 *                    Otherwise, defaults to true.
		 */
		if ( apply_filters( 'use_default_gallery_style', ! $html5 ) ) {
			$gallery_style = "
			<style type='text/css'>
				#{$selector} {
					margin: auto;
				}
				#{$selector} .gallery-item {
					float: {$float};
					margin-top: 10px;
					text-align: center;
					width: {$itemwidth}%;
				}
				#{$selector} img {
					border: 2px solid #cfcfcf;
				}
				#{$selector} .gallery-caption {
					margin-left: 0;
				}
				/* see gallery_shortcode() in wp-includes/media.php */
			</style>\n\t\t";
		}

		$size_class  = sanitize_html_class( $atts['size'] );
		$gallery_div = "<div id='$selector' class='gallery galleryid-{$id} gallery-columns-{$columns} gallery-size-{$size_class}'>";

		/**
		 * Filters the default gallery shortcode CSS styles.
		 *
		 * @since 2.5.0
		 *
		 * @param string $gallery_style Default CSS styles and opening HTML div container
		 *                              for the gallery shortcode output.
		 */
		$output = apply_filters( 'gallery_style', $gallery_style . $gallery_div );

		$i = 0;
		foreach ( $attachments as $id => $attachment ) {

			$attr = ( trim( $attachment->post_excerpt ) ) ? array( 'aria-describedby' => "$selector-$id" ) : '';
			if ( ! empty( $atts['link'] ) && 'file' === $atts['link'] ) {
				$image_output = wp_get_attachment_link( $id, $atts['size'], false, false, false, $attr );
			} elseif ( ! empty( $atts['link'] ) && 'none' === $atts['link'] ) {
				$image_output = wp_get_attachment_image( $id, $atts['size'], false, $attr );
			} else {
				$image_output = wp_get_attachment_link( $id, $atts['size'], true, false, false, $attr );
			}
			$image_meta = wp_get_attachment_metadata( $id );

			$orientation = '';
			if ( isset( $image_meta['height'], $image_meta['width'] ) ) {
				$orientation = ( $image_meta['height'] > $image_meta['width'] ) ? 'portrait' : 'landscape';
			}
			$output .= "<{$itemtag} class='gallery-item'>";
			$item_markup = "
				<{$icontag} class='gallery-icon {$orientation}'>
					$image_output
				</{$icontag}>";
			if ( $captiontag && trim( $attachment->post_excerpt ) ) {
				$item_caption = "
					<{$captiontag} class='wp-caption-text gallery-caption' id='$selector-$id'>
					" . wptexturize( $attachment->post_excerpt ) . "
					</{$captiontag}>";
				$item_markup  = str_replace( '</a>', $item_caption . '</a>', $item_markup );
			}
			$output .= $item_markup;

			$output .= "</{$itemtag}>";
			++ $i;
			if ( ! $html5 && $columns > 0 && 0 === $i % $columns ) {
				$output .= '<br style="clear: both" />';
			}
		}

		if ( ! $html5 && $columns > 0 && 0 !== $i % $columns ) {
			$output .= "
				<br style='clear: both' />";
		}

		$output .= "
			</div>\n";

		return $output;
	}

	/**
	 * Activate plugin.
	 *
	 * @return void
	 */
	public static function activate_plugin() {
		set_transient(
			'icdevops-gallery-caption-link-notice',
			__( '<a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=JJA37EHZXWUTJ" target="_blank" style="float:right; margin:20px"><img src="' . plugin_dir_url( __FILE__ ) . '/buy-me-a-coffee.png"></a><h3>Thank you for activating my plugin: WordPress Native Gallery Links on Captions!</h3>If you find the plugin useful and would like to support my work, please consider making a <a href="https://www.paypal.com/cgi-bin/webscr?cmd=_s-xclick&hosted_button_id=JJA37EHZXWUTJ" target="_blank">donation</a>.', 'ic-devops' )
		);
	}

	/**
	 * Deactivate plugin.
	 *
	 * @return void
	 */
	public static function deactivate_plugin() {
		self::admin_notices_cleanup( false );
	}

	/**
	 * Execute notices cleanup.
	 *
	 * @param  boolean $ajax Is AJAX call.
	 * @return void
	 */
	public static function admin_notices_cleanup( $ajax = true ) {
		// Delete transient, only display this notice once.
		delete_transient( 'icdevops-gallery-caption-link-notice' );

		if ( true === $ajax ) {
			// No need to continue.
			wp_die();
		}
	}

	/**
	 * Append the plugin URL.
	 *
	 * @param array $links The plugin links.
	 * @return array
	 */
	public static function plugin_action_links( $links ) {
		$links = array_merge(
			array(
				'<a href="https://iuliacazan.ro/apply-links-to-wordpress-native-gallery-captions/" target="_blank">' . __( 'Plugin URL', 'ic-devops' ) . '</a>',
			),
			$links
		);
		return $links;
	}

	/**
	 * Admin notices.
	 *
	 * @return void
	 */
	public static function admin_notices() {
		$maybe_trans = get_transient( 'icdevops-gallery-caption-link-notice' );
		if ( ! empty( $maybe_trans ) ) {
			?>
			<div class="updated notice is-dismissible fixgallink-notice" data-dismiss-url="<?php echo esc_url( admin_url( 'admin-ajax.php?action=icdevops-gallery-caption-link-deactivate-notice' ) ); ?>">
				<p><?php echo wp_kses_post( $maybe_trans ); ?></p>
			</div>
			<?php
		}
	}
}

// Instantiate the class.
$inst = ICDevOps_Fix_Gallery_Links::get_instance();
register_activation_hook( __FILE__, array( $inst, 'activate_plugin' ) );
register_deactivation_hook( __FILE__, array( $inst, 'deactivate_plugin' ) );
